class Module
  def cattr_accessor(attribute_name)
    class_eval <<-CODE
      def self.#{attribute_name}
        @@#{attribute_name} ||= nil
      end
      def self.#{attribute_name}=(value)
        @@#{attribute_name} = value
      end
    CODE
  end
end

class ElasticDot::Command::Base
  cattr_accessor :api

  protected
  def self.echo_off
    with_tty do
      system "stty -echo"
    end
  end

  def self.echo_on
    with_tty do
      system "stty echo"
    end
  end

  def self.with_tty(&block)
    return unless $stdin.isatty
    yield
  end

  def self.ask
    $stdin.gets.to_s.strip
  end

  def self.find_app!(opts)
    if opts[:app]
      @app = opts[:app]
    elsif app = extract_app_in_dir
      @app = app
    end

    return true if @app

    puts 'No app specified.'
    puts 'Specify which app to use with --app APP.'
    exit 1
  end

  def self.has_git?
    %x{ git --version }
    $?.success?
  end

  def self.git(args)
    return "" unless has_git?
    flattened_args = [args].flatten.compact.join(" ")
    %x{ git #{flattened_args} 2>&1 }.strip
  end

  def self.create_git_remote(remote, url)
    return if git('remote').split("\n").include?(remote)
    return unless File.exists?(".git")

    git "remote add #{remote} #{url}"

    puts "Git remote #{remote} added"
  end

  def self.which(cmd)
    exts = ENV['PATHEXT'] ? ENV['PATHEXT'].split(';') : ['']
    ENV['PATH'].split(File::PATH_SEPARATOR).each do |path|
      exts.each { |ext|
        exe = File.join(path, "#{cmd}#{ext}")
        return exe if File.executable? exe
      }
    end
    return nil
  end

  def self.extract_app_in_dir
    if app = extract_app_from_git_config
      return app
    elsif app = extract_app_from_git_config('elasticdot')
      return app
    else
      nil
    end
  end

  def self.extract_app_from_git_config(remote = 'origin')
    url = git "config remote.#{remote}.url"
    return nil if url.empty?

    if  url =~ /^git@git\.elasticdot\.com:([\w\d\-\.]+)\.git$/
      $1
    else
      nil
    end
  end

  def self.spinner(desc, &block)
    chars = %w{ | / - \\ }

    t = Thread.new { block.call }
    while t.alive?
      print "#{desc} [#{chars[0]}]"
      sleep 0.1
      print "\r"

      chars.push chars.shift
    end

    t.join

    puts "#{desc} [ok]"
  end
end
